/*	$Id: kvk.c,v 1.10 2000/04/11 17:09:41 bodo Exp $	*/
/*
    ChipDrive Library
    Copyright (C) 1997-2000  Bodo Bellut <bodo@bellutnet.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "common.h"

#ifndef lint
static char vcid[] UNUSED = "$Id: kvk.c,v 1.10 2000/04/11 17:09:41 bodo Exp $";
#endif /* lint */

#include <string.h>
#include <stdlib.h>


int kvk_read(int fd, t_kvk *kvk)
{
  /* Conversion table from `din66003' charset to `latin1' charset.
     Generated mechanically by GNU recode 3.4.

     The recoding should be reversible.
     */

  BYTE const din66003_to_latin1[256] =
  {
      0,   1,   2,   3,   4,   5,   6,   7,	/*   0 -   7 */
      8,   9,  10,  11,  12,  13,  14,  15,	/*   8 -  15 */
     16,  17,  18,  19,  20,  21,  22,  23,	/*  16 -  23 */
     24,  25,  26,  27,  28,  29,  30,  31,	/*  24 -  31 */
     32,  33,  34,  35,  36,  37,  38,  39,	/*  32 -  39 */
     40,  41,  42,  43,  44,  45,  46,  47,	/*  40 -  47 */
     48,  49,  50,  51,  52,  53,  54,  55,	/*  48 -  55 */
     56,  57,  58,  59,  60,  61,  62,  63,	/*  56 -  63 */
    167,  65,  66,  67,  68,  69,  70,  71,	/*  64 -  71 */
     72,  73,  74,  75,  76,  77,  78,  79,	/*  72 -  79 */
     80,  81,  82,  83,  84,  85,  86,  87,	/*  80 -  87 */
     88,  89,  90, 196, 214, 220,  94,  95,	/*  88 -  95 */
     96,  97,  98,  99, 100, 101, 102, 103,	/*  96 - 103 */
    104, 105, 106, 107, 108, 109, 110, 111,	/* 104 - 111 */
    112, 113, 114, 115, 116, 117, 118, 119,	/* 112 - 119 */
    120, 121, 122, 228, 246, 252, 223, 127,	/* 120 - 127 */
    128, 129, 130, 131, 132, 133, 134, 135,	/* 128 - 135 */
    136, 137, 138, 139, 140, 141, 142, 143,	/* 136 - 143 */
    144, 145, 146, 147, 148, 149, 150, 151,	/* 144 - 151 */
    152, 153, 154, 155, 156, 157, 158, 159,	/* 152 - 159 */
    160, 161, 162, 163, 164, 165, 166,  64,	/* 160 - 167 */
    168, 169, 170, 171, 172, 173, 174, 175,	/* 168 - 175 */
    176, 177, 178, 179, 180, 181, 182, 183,	/* 176 - 183 */
    184, 185, 186, 187, 188, 189, 190, 191,	/* 184 - 191 */
    192, 193, 194, 195,  91, 197, 198, 199,	/* 192 - 199 */
    200, 201, 202, 203, 204, 205, 206, 207,	/* 200 - 207 */
    208, 209, 210, 211, 212, 213,  92, 215,	/* 208 - 215 */
    216, 217, 218, 219,  93, 221, 222, 126,	/* 216 - 223 */
    224, 225, 226, 227, 123, 229, 230, 231,	/* 224 - 231 */
    232, 233, 234, 235, 236, 237, 238, 239,	/* 232 - 239 */
    240, 241, 242, 243, 244, 245, 124, 247,	/* 240 - 247 */
    248, 249, 250, 251, 125, 253, 254, 255,	/* 248 - 255 */
  };

  int i,j;
  BYTE *retval;
  BYTE *buf;
  BYTE status;
  int atr1 = 0;
  int prot = CD_PROT_UNKNOWN;
  
  retval = malloc(1*sizeof(BYTE));

  /* Status */
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, retval);
  status = *retval;
  read_cd(fd, retval);

  if (!((status >> 6) & 0x01)) {
    free(retval);
    return -ECD_NOCARD;
  }

  buf = malloc(256*sizeof(BYTE));
  
  bzero(buf, 256*sizeof(BYTE));

  bzero(kvk, sizeof(t_kvk));

  /* LED green */
  write_cd(fd, 5, 0x6f, 0x02, 0x6a, 0x0f, 0x5e);
  read_cd(fd, retval);

  /* activate */
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, retval);
  
  /* read i2c starting at 0 */
  qs = 0;
  write_qs(fd, 9, 0x7c, 0x64, 0x41, 0xa0, 0x00, 0x64, 0x40, 0xa1, 0x0f);
  write_qs(fd, 1, qs);
  read_cd(fd, retval);
  
  /* read 4 bytes */
  qs = 0;
  write_qs(fd, 1, 0x13);
  write_qs(fd, 1, qs);
  for (i=0; i < 4 ; i++) {
    read_qs(fd, retval);
    if (i==0)
      atr1 = *retval;
  }
  read_cd(fd, retval);
  
  /* ? */
  write_cd(fd, 4, 0x58, 0x65, 0x0f, 0x4e);
  read_cd(fd, retval);

  if (atr1 == 0xff) {
    
    /* read 2-wire starting at 0 */
    write_cd(fd, 9, 0x70, 0x64, 0x42, 0x30, 0x00, 0x00, 0x65, 0x0f, 0xa0);
    read_cd(fd, retval);
  
    /* read 4 bytes */
    qs = 0;
    write_qs(fd, 1, 0x13);
    write_qs(fd, 1, qs);
    for (i=0; i < 4 ; i++) {
      read_qs(fd, retval);
      if (i==0)
        atr1 = *retval;
    }
    read_cd(fd, retval);

    switch(atr1 & 0xf0) {
      case 0x80: prot = CD_PROT_I2C; break;
      case 0x90: prot = CD_PROT_3WIRE; break;
      case 0xa0: prot = CD_PROT_2WIRE; break;
    }

    /* ? */
    write_cd(fd, 4, 0xa0, 0x80, 0x0f, 0x1e);
    read_cd(fd, retval);
  } else
    prot = CD_PROT_I2C;
  
  if (prot == CD_PROT_2WIRE) {
    
    /* read 2-wire starting at 0 */
    write_cd(fd, 9, 0x70, 0x64, 0x42, 0x30, 0x00, 0x00, 0x65, 0x0f, 0xa0);
    read_cd(fd, retval);
    
    /* read 255 bytes */
    for (j=0; j < 17; j++) {
      qs = 0;
      write_qs(fd, 1, 0x1e);
      write_qs(fd, 1, qs);
      for (i=0; i < 15; i++)
        read_qs(fd, &buf[j*15+i]);
      read_cd(fd, retval);
    }
    
    /* read 1 byte */
    qs = 0;
    write_qs(fd, 1, 0x10);
    write_qs(fd, 1, qs);
    read_qs(fd, &buf[255]);
    read_cd(fd, retval);
  }

  /* deactivate */
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, retval);
  
  /* LED off */
  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, retval);

  free(retval);

  if (prot != CD_PROT_2WIRE) {
    free(buf);
    return -ECD_NOSYS;
  }
  
  i = 0x20;
  do {
    int len;
    int tag;
    char tmpbuf[29];
    
    tag=buf[i++];
    len=buf[i++];
    if (tag != 0xc0) {
      bzero(tmpbuf, 30*sizeof(char));
      for (j=0; j < len; j++)
	tmpbuf[j] = din66003_to_latin1[buf[i++]];

      len++;
      
      if (tag >= 0x80)
	switch (tag) {
	case 0x80: strncpy(kvk->kkName,   tmpbuf, len); break;
	case 0x81: strncpy(kvk->kkNum,    tmpbuf, len); break;
	case 0x8f: strncpy(kvk->VKNR,     tmpbuf, len); break;
	case 0x82: strncpy(kvk->vNum,     tmpbuf, len); break;
	case 0x83: strncpy(kvk->vStat,    tmpbuf, len); break;
	case 0x90: strncpy(kvk->statOW,   tmpbuf, len); break;
	case 0x84: strncpy(kvk->titel,    tmpbuf, len); break;
	case 0x85: strncpy(kvk->vorname,  tmpbuf, len); break;
	case 0x86: strncpy(kvk->namZus,   tmpbuf, len); break;
	case 0x87: strncpy(kvk->famName,  tmpbuf, len); break;
	case 0x88: strncpy(kvk->gebDat,   tmpbuf, len); break;
	case 0x89: strncpy(kvk->strNum,   tmpbuf, len); break;
	case 0x8a: strncpy(kvk->wohnSLC,  tmpbuf, len); break;
	case 0x8b: strncpy(kvk->PLZ,      tmpbuf, len); break;
	case 0x8c: strncpy(kvk->ort,      tmpbuf, len); break;
	case 0x8d: strncpy(kvk->gueltig,  tmpbuf, len); break;
	case 0x8e: strncpy(&kvk->checksum, tmpbuf, len); break;
	case 0xc0: break;
	}

    } else
      i+=len;
  } while (i < 255);

  free(buf);

  return 0;
  
}
