/*	$Id: tele.c,v 1.8 2000/04/11 17:09:41 bodo Exp $	*/
/*
    ChipDrive Library
    Copyright (C) 1997-2000  Bodo Bellut <bodo@bellutnet.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "common.h"


#ifndef lint
static char vcid[] UNUSED = "$Id: tele.c,v 1.8 2000/04/11 17:09:41 bodo Exp $";
#endif /* lint */


const char *tele_manfct[] = {"Orga Kartensysteme",
			     "Giesecke & Devrient",
			     "Oldenbourg Datensysteme",
			     "Gemplus",
			     "Solaic",
			     "Uniqa",
			     "Schlumberger"};

const char *tele_code[] = {"", "", "", "1,50 DM",
			   "6,00 DM", "12,00 DM",
			   "50,00/60,00 DM"};


int count_bits(const BYTE buf)
{
  int tmp=0,i;
  
  for (i=0; i < 8; i++)
    if ((buf >> i) & 1)
      tmp++;
      
  return tmp;
}


int tele_read(int fd, t_tele *tele)
{
  int i;
  BYTE retval;
  BYTE buf[13];
  BYTE status;

  bzero(buf, 13*sizeof(BYTE));
  bzero(tele, sizeof(t_tele));

  /* Status */
  write_cd(fd, 2, 0x03, 0x07);
  read_cd(fd, &status);
  read_cd(fd, &retval);
  
  if (!((status >> 6) & 0x01)) {
    return -ECD_NOCARD;
  }

  /* LED green */
  write_cd(fd, 5, 0x6f, 0x02, 0x6a, 0x0f, 0x5e);
  read_cd(fd, &retval);

  /* activate */
  write_cd(fd, 3, 0x60, 0x0f, 0x9c);
  read_cd(fd, &retval);
  
  /* ? */
  qs = 0;
  write_qs(fd, 4, 0x70, 0x62, 0x13, 0x2b);
  read_qs(fd, &retval);
  read_qs(fd, &retval);
  read_qs(fd, &retval);
  read_qs(fd, &retval);
  read_cd(fd, &retval);
  
  /* ? */
  write_cd(fd, 4, 0x70, 0x62, 0x0f, 0x13);
  read_cd(fd, &retval);
  
  /* 13 bytes lesen */
  qs = 0;
  write_qs(fd, 1, 0x1c);
  write_qs(fd, 1, qs);
  for (i=0; i < 13 ; i++)
    read_qs(fd, &buf[i]);
  read_cd(fd, &retval);

  /* deactivate */
  write_cd(fd, 3, 0x61, 0x0f, 0x98);
  read_cd(fd, &retval);
  
  /* LED off */
  write_cd(fd, 3, 0x6b, 0x0f, 0xb0);
  read_cd(fd, &retval);
  
  tele->startup = ((buf[0] & 0xff) << 8) | (buf[1] & 0xff);
  tele->manfct  = (buf[3] & 0x0f);
  tele->code    = (buf[4] & 0x0f);
  tele->monat   = (buf[5] & 0x0f);
  tele->jahr4   = (buf[4] & 0xf0) >> 4;
  sprintf(tele->serNum, "%d%d%d%d%d", (buf[7] & 0xf0) >> 4, buf[7] & 0x0f,
	                              (buf[6] & 0xf0) >> 4, buf[6] & 0x0f,
	                              (buf[5] & 0xf0) >> 4);
  tele->wert = count_bits(buf[ 8]) * 4096 +
               count_bits(buf[ 9]) *  512 +
               count_bits(buf[10]) *   64 +
               count_bits(buf[11]) *    8 +
               count_bits(buf[12]);

  return 0;
}
